# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from .types.query_investigation_response import QueryInvestigationResponse
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.not_implemented_error import NotImplementedError
from ..common.errors.bad_gateway_error import BadGatewayError
from ..common.errors.service_unavailable_error import ServiceUnavailableError
from ..common.errors.gateway_timeout_error import GatewayTimeoutError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.get_investigation_response import GetInvestigationResponse
from ..core.jsonable_encoder import jsonable_encoder
from .types.patch_investigation_request import PatchInvestigationRequest
from .types.get_evidence_response import GetEvidenceResponse
from .types.query_log_providers_response import QueryLogProvidersResponse
from ..events.types.event import Event
from .types.create_siem_events_response import CreateSiemEventsResponse
from .types.query_siem_events_response import QuerySiemEventsResponse
from .types.query_siem_alerts_response import QuerySiemAlertsResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SiemClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def query_investigations(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryInvestigationResponse:
        """
        Queries investigations

        Parameters
        ----------
        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of `Investigation` objects to return in this page. Defaults to 100.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryInvestigationResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.siem.query_investigations(
            meta="string",
            cursor="string",
            limit=1,
            order="string",
            filter="string",
            include_raw_data=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/siem/investigations",
            method="GET",
            params={
                "meta": meta,
                "cursor": cursor,
                "limit": limit,
                "order": order,
                "filter": filter,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QueryInvestigationResponse,
                    construct_type(
                        type_=QueryInvestigationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_investigation(
        self,
        id: str,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetInvestigationResponse:
        """
        Retrieves an investigation by ID.

        Parameters
        ----------
        id : str
            ID of the investigation to retrieve.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInvestigationResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.siem.get_investigation(
            id="string",
            meta="string",
            include_raw_data=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/siem/investigations/{jsonable_encoder(id)}",
            method="GET",
            params={
                "meta": meta,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInvestigationResponse,
                    construct_type(
                        type_=GetInvestigationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch_investigation(
        self,
        id: str,
        *,
        request: PatchInvestigationRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetInvestigationResponse:
        """
        Updates an investigation by ID.

        Parameters
        ----------
        id : str
            ID of the investigation to update.

        request : PatchInvestigationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInvestigationResponse

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.common import PatchOp, PatchOperation

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.siem.patch_investigation(
            id="string",
            request=[
                PatchOperation(
                    op=PatchOp.ADD,
                    path="string",
                    from_="string",
                    value={"key": "value"},
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/siem/investigations/{jsonable_encoder(id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInvestigationResponse,
                    construct_type(
                        type_=GetInvestigationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_evidence(
        self,
        id: str,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEvidenceResponse:
        """
        Retrieves the evidence for an investigation.

        Parameters
        ----------
        id : str
            ID of the investigation to retrieve evidence for.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEvidenceResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.siem.get_evidence(
            id="string",
            meta="string",
            include_raw_data=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/siem/investigations/{jsonable_encoder(id)}/evidence",
            method="GET",
            params={
                "meta": meta,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetEvidenceResponse,
                    construct_type(
                        type_=GetEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def query_log_providers(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryLogProvidersResponse:
        """
        Queries available log providers in the source SIEM

        Parameters
        ----------
        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of log provider objects to return in this page. Defaults to 100.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryLogProvidersResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.siem.query_log_providers(
            meta="string",
            cursor="string",
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/siem/log-providers",
            method="GET",
            params={
                "meta": meta,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QueryLogProvidersResponse,
                    construct_type(
                        type_=QueryLogProvidersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def post_events(
        self,
        *,
        request: typing.Sequence[Event],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSiemEventsResponse:
        """
        Writes a batch of `Event` objects to the SIEM configured with the token used for authentication.

        Parameters
        ----------
        request : typing.Sequence[Event]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSiemEventsResponse

        Examples
        --------
        import datetime

        from synqly import SynqlyEngine
        from synqly.events import Event_AccountChange
        from synqly.ocsf.v_1_3_0.accountchange.objects import (
            Account,
            Actor,
            Api,
            AutonomousSystem,
            Cloud,
            Container,
            Device,
            DeviceHwInfo,
            Enrichment,
            Extension,
            Feature,
            Group,
            HttpRequest,
            Idp,
            Image,
            Location,
            Logger,
            Metadata,
            NetworkEndpoint,
            NetworkProxy,
            Observable,
            Organization,
            Os,
            Osint,
            Policy,
            Process,
            Product,
            Request,
            Response,
            Service,
            Session,
            Url,
            User,
        )

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.siem.post_events(
            request=[
                Event_AccountChange(
                    activity_id=1,
                    activity_name="string",
                    actor=Actor(
                        actor_type="string",
                        actor_type_id=1,
                        app_name="string",
                        app_uid="string",
                        authorizations=[],
                        groups=[],
                        idp=Idp(),
                        invoked_by="string",
                        process=Process(),
                        session=Session(),
                        user=User(),
                    ),
                    api=Api(
                        group=Group(),
                        operation="string",
                        request=Request(
                            uid="string",
                        ),
                        response=Response(),
                        service=Service(),
                        version="string",
                    ),
                    category_name="string",
                    category_uid=1,
                    class_uid=1,
                    cloud=Cloud(
                        account=Account(),
                        org=Organization(),
                        project_uid="string",
                        provider="string",
                        region="string",
                        zone="string",
                    ),
                    count=1,
                    custom_fields={"string": {"key": "value"}},
                    device=Device(
                        agent_list=[],
                        autoscale_uid="string",
                        boot_time=1,
                        boot_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        container=Container(),
                        created_time=1,
                        created_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        desc="string",
                        domain="string",
                        first_seen_time=1,
                        first_seen_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        groups=[],
                        hostname="string",
                        hw_info=DeviceHwInfo(),
                        hypervisor="string",
                        image=Image(
                            uid="string",
                        ),
                        imei="string",
                        instance_uid="string",
                        interface_name="string",
                        interface_uid="string",
                        ip="string",
                        ip_addresses=[],
                        is_compliant=True,
                        is_managed=True,
                        is_personal=True,
                        is_trusted=True,
                        last_seen_time=1,
                        last_seen_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        location=Location(),
                        mac="string",
                        mac_addresses=[],
                        modified_time=1,
                        modified_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        name="string",
                        namespace_pid=1,
                        netbios_names=[],
                        network_interfaces=[],
                        network_status="string",
                        network_status_id=1,
                        org=Organization(),
                        os=Os(
                            name="string",
                            type_id=1,
                        ),
                        owner=User(),
                        region="string",
                        risk_level="string",
                        risk_level_id=1,
                        risk_score=1,
                        subnet="string",
                        subnet_uid="string",
                        sw_info=[],
                        type="string",
                        type_id=1,
                        uid="string",
                        uid_alt="string",
                        vendor=Organization(),
                        vlan_uid="string",
                        vpc_uid="string",
                        zone="string",
                    ),
                    duration=1,
                    end_time=1,
                    end_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    enrichments=[
                        Enrichment(
                            data={"key": "value"},
                            name="string",
                            value="string",
                        )
                    ],
                    http_request=HttpRequest(
                        args="string",
                        http_headers=[],
                        http_method="string",
                        length=1,
                        referrer="string",
                        uid="string",
                        url=Url(),
                        user_agent="string",
                        user_agent_name="string",
                        version="string",
                        x_forwarded_for=[],
                    ),
                    message="string",
                    metadata=Metadata(
                        correlation_uid="string",
                        event_code="string",
                        extension=Extension(
                            name="string",
                            uid="string",
                            version="string",
                        ),
                        extensions=[
                            Extension(
                                name="string",
                                uid="string",
                                version="string",
                            )
                        ],
                        labels=["string"],
                        log_level="string",
                        log_name="string",
                        log_provider="string",
                        log_version="string",
                        logged_time=1,
                        logged_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        loggers=[Logger()],
                        modified_time=1,
                        modified_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        original_time="string",
                        processed_time=1,
                        processed_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        product=Product(
                            cpe_name="string",
                            distribution_mode="string",
                            feature=Feature(
                                name="string",
                                uid="string",
                                version="string",
                            ),
                            lang="string",
                            name="string",
                            path="string",
                            uid="string",
                            url_string="string",
                            vendor_name="string",
                            version="string",
                            workload="string",
                        ),
                        profiles=["string"],
                        sequence=1,
                        tenant_uid="string",
                        uid="string",
                        version="string",
                    ),
                    observables=[
                        Observable(
                            name="string",
                            type_id=1,
                        )
                    ],
                    osint=[
                        Osint(
                            type_id=1,
                            value="string",
                        )
                    ],
                    policy=Policy(
                        desc="string",
                        group=Group(),
                        is_applied=True,
                        name="string",
                        uid="string",
                        version="string",
                    ),
                    raw_data="string",
                    severity="string",
                    severity_id=1,
                    src_endpoint=NetworkEndpoint(
                        agent_list=[],
                        autonomous_system=AutonomousSystem(),
                        container=Container(),
                        domain="string",
                        hostname="string",
                        hw_info=DeviceHwInfo(),
                        instance_uid="string",
                        interface_name="string",
                        interface_uid="string",
                        intermediate_ips=[],
                        ip="string",
                        location=Location(),
                        mac="string",
                        name="string",
                        namespace_pid=1,
                        os=Os(
                            name="string",
                            type_id=1,
                        ),
                        owner=User(),
                        port=1,
                        proxy_endpoint=NetworkProxy(),
                        subnet_uid="string",
                        svc_name="string",
                        type="string",
                        type_id=1,
                        uid="string",
                        vlan_uid="string",
                        vpc_uid="string",
                        zone="string",
                    ),
                    start_time=1,
                    start_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    status="string",
                    status_code="string",
                    status_detail="string",
                    status_id=1,
                    time=1,
                    time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    timezone_offset=1,
                    type_name="string",
                    type_uid=1,
                    unmapped={"string": {"key": "value"}},
                    user=User(),
                    user_result=User(),
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/siem/events",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateSiemEventsResponse,
                    construct_type(
                        type_=CreateSiemEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def query_events(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        passthrough_param: typing.Optional[
            typing.Union[str, typing.Sequence[str]]
        ] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QuerySiemEventsResponse:
        """
        Queries events from the SIEM configured with the token used for authentication.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of `Account` objects to return in this page. Defaults to 100.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `time`. To control the direction of the sorting, append `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order. The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter. If used more than once, the queries are ANDed together.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        passthrough_param : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Provider-specific query to pass through to the SIEM. This is useful for advanced queries that are not
            supported by the API. The keys and values are provider-specific. For example, to perform a specific
            query in Rapid7 IDR, you can use the `query: "{advanced query}"` key-value pair.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. This is useful for debugging and troubleshooting. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QuerySiemEventsResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.siem.query_events(
            cursor="string",
            limit=1,
            order="string",
            filter="string",
            meta="string",
            passthrough_param="string",
            include_raw_data=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/siem/events",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
                "order": order,
                "filter": filter,
                "meta": meta,
                "passthrough-param": passthrough_param,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QuerySiemEventsResponse,
                    construct_type(
                        type_=QuerySiemEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def query_alerts(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QuerySiemAlertsResponse:
        """
        Queries alerts from the SIEM configured with the token used for authentication.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of `Account` objects to return in this page. Defaults to 100.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `time`. To control the direction of the sorting, append `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order. The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter. If used more than once, the queries are ANDed together.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. This is useful for debugging and troubleshooting. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QuerySiemAlertsResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.siem.query_alerts(
            cursor="string",
            limit=1,
            order="string",
            filter="string",
            meta="string",
            include_raw_data=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/siem/alerts",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
                "order": order,
                "filter": filter,
                "meta": meta,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QuerySiemAlertsResponse,
                    construct_type(
                        type_=QuerySiemAlertsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSiemClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def query_investigations(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryInvestigationResponse:
        """
        Queries investigations

        Parameters
        ----------
        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of `Investigation` objects to return in this page. Defaults to 100.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryInvestigationResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.siem.query_investigations(
                meta="string",
                cursor="string",
                limit=1,
                order="string",
                filter="string",
                include_raw_data=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/siem/investigations",
            method="GET",
            params={
                "meta": meta,
                "cursor": cursor,
                "limit": limit,
                "order": order,
                "filter": filter,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QueryInvestigationResponse,
                    construct_type(
                        type_=QueryInvestigationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_investigation(
        self,
        id: str,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetInvestigationResponse:
        """
        Retrieves an investigation by ID.

        Parameters
        ----------
        id : str
            ID of the investigation to retrieve.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInvestigationResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.siem.get_investigation(
                id="string",
                meta="string",
                include_raw_data=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/siem/investigations/{jsonable_encoder(id)}",
            method="GET",
            params={
                "meta": meta,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInvestigationResponse,
                    construct_type(
                        type_=GetInvestigationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch_investigation(
        self,
        id: str,
        *,
        request: PatchInvestigationRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetInvestigationResponse:
        """
        Updates an investigation by ID.

        Parameters
        ----------
        id : str
            ID of the investigation to update.

        request : PatchInvestigationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInvestigationResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.common import PatchOp, PatchOperation

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.siem.patch_investigation(
                id="string",
                request=[
                    PatchOperation(
                        op=PatchOp.ADD,
                        path="string",
                        from_="string",
                        value={"key": "value"},
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/siem/investigations/{jsonable_encoder(id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInvestigationResponse,
                    construct_type(
                        type_=GetInvestigationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_evidence(
        self,
        id: str,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEvidenceResponse:
        """
        Retrieves the evidence for an investigation.

        Parameters
        ----------
        id : str
            ID of the investigation to retrieve evidence for.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEvidenceResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.siem.get_evidence(
                id="string",
                meta="string",
                include_raw_data=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/siem/investigations/{jsonable_encoder(id)}/evidence",
            method="GET",
            params={
                "meta": meta,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetEvidenceResponse,
                    construct_type(
                        type_=GetEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def query_log_providers(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryLogProvidersResponse:
        """
        Queries available log providers in the source SIEM

        Parameters
        ----------
        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of log provider objects to return in this page. Defaults to 100.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryLogProvidersResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.siem.query_log_providers(
                meta="string",
                cursor="string",
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/siem/log-providers",
            method="GET",
            params={
                "meta": meta,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QueryLogProvidersResponse,
                    construct_type(
                        type_=QueryLogProvidersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def post_events(
        self,
        *,
        request: typing.Sequence[Event],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSiemEventsResponse:
        """
        Writes a batch of `Event` objects to the SIEM configured with the token used for authentication.

        Parameters
        ----------
        request : typing.Sequence[Event]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSiemEventsResponse

        Examples
        --------
        import asyncio
        import datetime

        from synqly import AsyncSynqlyEngine
        from synqly.events import Event_AccountChange
        from synqly.ocsf.v_1_3_0.accountchange.objects import (
            Account,
            Actor,
            Api,
            AutonomousSystem,
            Cloud,
            Container,
            Device,
            DeviceHwInfo,
            Enrichment,
            Extension,
            Feature,
            Group,
            HttpRequest,
            Idp,
            Image,
            Location,
            Logger,
            Metadata,
            NetworkEndpoint,
            NetworkProxy,
            Observable,
            Organization,
            Os,
            Osint,
            Policy,
            Process,
            Product,
            Request,
            Response,
            Service,
            Session,
            Url,
            User,
        )

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.siem.post_events(
                request=[
                    Event_AccountChange(
                        activity_id=1,
                        activity_name="string",
                        actor=Actor(
                            actor_type="string",
                            actor_type_id=1,
                            app_name="string",
                            app_uid="string",
                            authorizations=[],
                            groups=[],
                            idp=Idp(),
                            invoked_by="string",
                            process=Process(),
                            session=Session(),
                            user=User(),
                        ),
                        api=Api(
                            group=Group(),
                            operation="string",
                            request=Request(
                                uid="string",
                            ),
                            response=Response(),
                            service=Service(),
                            version="string",
                        ),
                        category_name="string",
                        category_uid=1,
                        class_uid=1,
                        cloud=Cloud(
                            account=Account(),
                            org=Organization(),
                            project_uid="string",
                            provider="string",
                            region="string",
                            zone="string",
                        ),
                        count=1,
                        custom_fields={"string": {"key": "value"}},
                        device=Device(
                            agent_list=[],
                            autoscale_uid="string",
                            boot_time=1,
                            boot_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            container=Container(),
                            created_time=1,
                            created_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            desc="string",
                            domain="string",
                            first_seen_time=1,
                            first_seen_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            groups=[],
                            hostname="string",
                            hw_info=DeviceHwInfo(),
                            hypervisor="string",
                            image=Image(
                                uid="string",
                            ),
                            imei="string",
                            instance_uid="string",
                            interface_name="string",
                            interface_uid="string",
                            ip="string",
                            ip_addresses=[],
                            is_compliant=True,
                            is_managed=True,
                            is_personal=True,
                            is_trusted=True,
                            last_seen_time=1,
                            last_seen_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            location=Location(),
                            mac="string",
                            mac_addresses=[],
                            modified_time=1,
                            modified_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            name="string",
                            namespace_pid=1,
                            netbios_names=[],
                            network_interfaces=[],
                            network_status="string",
                            network_status_id=1,
                            org=Organization(),
                            os=Os(
                                name="string",
                                type_id=1,
                            ),
                            owner=User(),
                            region="string",
                            risk_level="string",
                            risk_level_id=1,
                            risk_score=1,
                            subnet="string",
                            subnet_uid="string",
                            sw_info=[],
                            type="string",
                            type_id=1,
                            uid="string",
                            uid_alt="string",
                            vendor=Organization(),
                            vlan_uid="string",
                            vpc_uid="string",
                            zone="string",
                        ),
                        duration=1,
                        end_time=1,
                        end_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        enrichments=[
                            Enrichment(
                                data={"key": "value"},
                                name="string",
                                value="string",
                            )
                        ],
                        http_request=HttpRequest(
                            args="string",
                            http_headers=[],
                            http_method="string",
                            length=1,
                            referrer="string",
                            uid="string",
                            url=Url(),
                            user_agent="string",
                            user_agent_name="string",
                            version="string",
                            x_forwarded_for=[],
                        ),
                        message="string",
                        metadata=Metadata(
                            correlation_uid="string",
                            event_code="string",
                            extension=Extension(
                                name="string",
                                uid="string",
                                version="string",
                            ),
                            extensions=[
                                Extension(
                                    name="string",
                                    uid="string",
                                    version="string",
                                )
                            ],
                            labels=["string"],
                            log_level="string",
                            log_name="string",
                            log_provider="string",
                            log_version="string",
                            logged_time=1,
                            logged_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            loggers=[Logger()],
                            modified_time=1,
                            modified_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            original_time="string",
                            processed_time=1,
                            processed_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            product=Product(
                                cpe_name="string",
                                distribution_mode="string",
                                feature=Feature(
                                    name="string",
                                    uid="string",
                                    version="string",
                                ),
                                lang="string",
                                name="string",
                                path="string",
                                uid="string",
                                url_string="string",
                                vendor_name="string",
                                version="string",
                                workload="string",
                            ),
                            profiles=["string"],
                            sequence=1,
                            tenant_uid="string",
                            uid="string",
                            version="string",
                        ),
                        observables=[
                            Observable(
                                name="string",
                                type_id=1,
                            )
                        ],
                        osint=[
                            Osint(
                                type_id=1,
                                value="string",
                            )
                        ],
                        policy=Policy(
                            desc="string",
                            group=Group(),
                            is_applied=True,
                            name="string",
                            uid="string",
                            version="string",
                        ),
                        raw_data="string",
                        severity="string",
                        severity_id=1,
                        src_endpoint=NetworkEndpoint(
                            agent_list=[],
                            autonomous_system=AutonomousSystem(),
                            container=Container(),
                            domain="string",
                            hostname="string",
                            hw_info=DeviceHwInfo(),
                            instance_uid="string",
                            interface_name="string",
                            interface_uid="string",
                            intermediate_ips=[],
                            ip="string",
                            location=Location(),
                            mac="string",
                            name="string",
                            namespace_pid=1,
                            os=Os(
                                name="string",
                                type_id=1,
                            ),
                            owner=User(),
                            port=1,
                            proxy_endpoint=NetworkProxy(),
                            subnet_uid="string",
                            svc_name="string",
                            type="string",
                            type_id=1,
                            uid="string",
                            vlan_uid="string",
                            vpc_uid="string",
                            zone="string",
                        ),
                        start_time=1,
                        start_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        status="string",
                        status_code="string",
                        status_detail="string",
                        status_id=1,
                        time=1,
                        time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        timezone_offset=1,
                        type_name="string",
                        type_uid=1,
                        unmapped={"string": {"key": "value"}},
                        user=User(),
                        user_result=User(),
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/siem/events",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateSiemEventsResponse,
                    construct_type(
                        type_=CreateSiemEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def query_events(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        passthrough_param: typing.Optional[
            typing.Union[str, typing.Sequence[str]]
        ] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QuerySiemEventsResponse:
        """
        Queries events from the SIEM configured with the token used for authentication.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of `Account` objects to return in this page. Defaults to 100.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `time`. To control the direction of the sorting, append `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order. The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter. If used more than once, the queries are ANDed together.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        passthrough_param : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Provider-specific query to pass through to the SIEM. This is useful for advanced queries that are not
            supported by the API. The keys and values are provider-specific. For example, to perform a specific
            query in Rapid7 IDR, you can use the `query: "{advanced query}"` key-value pair.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. This is useful for debugging and troubleshooting. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QuerySiemEventsResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.siem.query_events(
                cursor="string",
                limit=1,
                order="string",
                filter="string",
                meta="string",
                passthrough_param="string",
                include_raw_data=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/siem/events",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
                "order": order,
                "filter": filter,
                "meta": meta,
                "passthrough-param": passthrough_param,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QuerySiemEventsResponse,
                    construct_type(
                        type_=QuerySiemEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def query_alerts(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QuerySiemAlertsResponse:
        """
        Queries alerts from the SIEM configured with the token used for authentication.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of `Account` objects to return in this page. Defaults to 100.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `time`. To control the direction of the sorting, append `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order. The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter. If used more than once, the queries are ANDed together.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        include_raw_data : typing.Optional[bool]
            Include the raw data from the SIEM in the response. This is useful for debugging and troubleshooting. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QuerySiemAlertsResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.siem.query_alerts(
                cursor="string",
                limit=1,
                order="string",
                filter="string",
                meta="string",
                include_raw_data=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/siem/alerts",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
                "order": order,
                "filter": filter,
                "meta": meta,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QuerySiemAlertsResponse,
                    construct_type(
                        type_=QuerySiemAlertsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    GetGroupMembersResponse,
    GetGroupResponse,
    GetUserResponse,
    GroupId,
    QueryGroupsResponse,
    QueryIdentityAuditLogResponse,
    QueryUsersResponse,
    UserId,
)

__all__ = [
    "GetGroupMembersResponse",
    "GetGroupResponse",
    "GetUserResponse",
    "GroupId",
    "QueryGroupsResponse",
    "QueryIdentityAuditLogResponse",
    "QueryUsersResponse",
    "UserId",
]



# This file was auto-generated by Fern from our API Definition.

from .appsec_amazon_inspector import AppsecAmazonInspector
from .appsec_git_lab import AppsecGitLab
from .appsec_hcl_app_scan_on_cloud import AppsecHclAppScanOnCloud
from .appsec_open_text_application_security import AppsecOpenTextApplicationSecurity
from .appsec_open_text_core_application_security import (
    AppsecOpenTextCoreApplicationSecurity,
)
from .appsec_open_text_core_application_security_mock import (
    AppsecOpenTextCoreApplicationSecurityMock,
)
from .appsec_opentext_core_application_security_dataset import (
    AppsecOpentextCoreApplicationSecurityDataset,
)
from .armis_credential import (
    ArmisCredential,
    ArmisCredential_Token,
    ArmisCredential_TokenId,
)
from .assets_armis_centrix import AssetsArmisCentrix
from .assets_armis_centrix_mock import AssetsArmisCentrixMock
from .assets_armis_dataset import AssetsArmisDataset
from .assets_axonius import AssetsAxonius
from .assets_axonius_dataset import AssetsAxoniusDataset
from .assets_axonius_mock import AssetsAxoniusMock
from .assets_claroty_xdome import AssetsClarotyXdome
from .assets_crowd_strike import AssetsCrowdStrike
from .assets_crowd_strike_dataset import AssetsCrowdStrikeDataset
from .assets_crowd_strike_mock import AssetsCrowdStrikeMock
from .assets_ivanti_neurons import AssetsIvantiNeurons
from .assets_nozomi_vantage import AssetsNozomiVantage
from .assets_nozomi_vantage_dataset import AssetsNozomiVantageDataset
from .assets_nozomi_vantage_mock import AssetsNozomiVantageMock
from .assets_qualys_cloud import AssetsQualysCloud
from .assets_qualys_cloud_dataset import AssetsQualysCloudDataset
from .assets_qualys_cloud_mock import AssetsQualysCloudMock
from .assets_service_now import AssetsServiceNow
from .assets_service_now_dataset import AssetsServiceNowDataset
from .assets_service_now_mock import AssetsServiceNowMock
from .assets_sevco import AssetsSevco
from .assets_sevco_dataset import AssetsSevcoDataset
from .assets_sevco_mock import AssetsSevcoMock
from .assets_tanium_cloud import AssetsTaniumCloud
from .assets_tanium_cloud_dataset import AssetsTaniumCloudDataset
from .assets_tanium_cloud_mock import AssetsTaniumCloudMock
from .autotask_api_integration_code_credential import (
    AutotaskApiIntegrationCodeCredential,
    AutotaskApiIntegrationCodeCredential_Secret,
    AutotaskApiIntegrationCodeCredential_SecretId,
)
from .autotask_secret_credential import (
    AutotaskSecretCredential,
    AutotaskSecretCredential_Secret,
    AutotaskSecretCredential_SecretId,
)
from .aws_provider_credential import (
    AwsProviderCredential,
    AwsProviderCredential_Aws,
    AwsProviderCredential_AwsId,
)
from .aws_region import AwsRegion
from .aws_s_3_credential import (
    AwsS3Credential,
    AwsS3Credential_Aws,
    AwsS3Credential_AwsId,
)
from .aws_security_lake_credential import (
    AwsSecurityLakeCredential,
    AwsSecurityLakeCredential_Aws,
    AwsSecurityLakeCredential_AwsId,
)
from .aws_sqs_credential import (
    AwsSqsCredential,
    AwsSqsCredential_Aws,
    AwsSqsCredential_AwsId,
)
from .axonius_credential import (
    AxoniusCredential,
    AxoniusCredential_Basic,
    AxoniusCredential_BasicId,
)
from .azure_blob_credential import (
    AzureBlobCredential,
    AzureBlobCredential_Token,
    AzureBlobCredential_TokenId,
)
from .azure_monitor_logs_credential import (
    AzureMonitorLogsCredential,
    AzureMonitorLogsCredential_Token,
    AzureMonitorLogsCredential_TokenId,
)
from .claroty_apiurl import ClarotyApiurl
from .claroty_credential import (
    ClarotyCredential,
    ClarotyCredential_Token,
    ClarotyCredential_TokenId,
)
from .cloud_security_aws import CloudSecurityAws
from .cloud_security_crowd_strike import CloudSecurityCrowdStrike
from .cloud_security_defender import CloudSecurityDefender
from .cloud_security_palo_alto import CloudSecurityPaloAlto
from .crowd_strike_credential import (
    CrowdStrikeCredential,
    CrowdStrikeCredential_OAuthClient,
    CrowdStrikeCredential_OAuthClientId,
)
from .crowdstrike_hec_credential import (
    CrowdstrikeHecCredential,
    CrowdstrikeHecCredential_Token,
    CrowdstrikeHecCredential_TokenId,
)
from .custom_credential import (
    CustomCredential,
    CustomCredential_Aws,
    CustomCredential_AwsId,
    CustomCredential_Basic,
    CustomCredential_BasicId,
    CustomCredential_OAuthClient,
    CustomCredential_OAuthClientId,
    CustomCredential_Secret,
    CustomCredential_SecretId,
    CustomCredential_Token,
    CustomCredential_TokenId,
)
from .custom_endpoint import CustomEndpoint
from .custom_field_mapping import CustomFieldMapping
from .custom_synqly import CustomSynqly
from .defender_credential import (
    DefenderCredential,
    DefenderCredential_OAuthClient,
    DefenderCredential_OAuthClientId,
)
from .edr_crowd_strike import EdrCrowdStrike
from .edr_crowd_strike_dataset import EdrCrowdStrikeDataset
from .edr_crowd_strike_mock import EdrCrowdStrikeMock
from .edr_defender import EdrDefender
from .edr_malwarebytes import EdrMalwarebytes
from .edr_sentinel_one import EdrSentinelOne
from .edr_sophos import EdrSophos
from .edr_tanium import EdrTanium
from .elasticsearch_auth_options import ElasticsearchAuthOptions
from .elasticsearch_credential import (
    ElasticsearchCredential,
    ElasticsearchCredential_Basic,
    ElasticsearchCredential_BasicId,
    ElasticsearchCredential_OAuthClient,
    ElasticsearchCredential_OAuthClientId,
    ElasticsearchCredential_Token,
    ElasticsearchCredential_TokenId,
)
from .elasticsearch_shared_secret import (
    ElasticsearchSharedSecret,
    ElasticsearchSharedSecret_Secret,
    ElasticsearchSharedSecret_SecretId,
)
from .entra_id_credential import (
    EntraIdCredential,
    EntraIdCredential_OAuthClient,
    EntraIdCredential_OAuthClientId,
)
from .freshdesk_credential import (
    FreshdeskCredential,
    FreshdeskCredential_Token,
    FreshdeskCredential_TokenId,
)
from .gcs_credential import GcsCredential, GcsCredential_Aws, GcsCredential_AwsId
from .gcs_json_credential import (
    GcsJsonCredential,
    GcsJsonCredential_Token,
    GcsJsonCredential_TokenId,
)
from .git_lab_credential import (
    GitLabCredential,
    GitLabCredential_Token,
    GitLabCredential_TokenId,
)
from .google_chronicle_credential import (
    GoogleChronicleCredential,
    GoogleChronicleCredential_OAuthClient,
    GoogleChronicleCredential_OAuthClientId,
)
from .google_credential import (
    GoogleCredential,
    GoogleCredential_OAuthClient,
    GoogleCredential_OAuthClientId,
)
from .google_service_account_credential import (
    GoogleServiceAccountCredential,
    GoogleServiceAccountCredential_OAuthClient,
    GoogleServiceAccountCredential_OAuthClientId,
)
from .hcl_app_scan_on_cloud_credential import (
    HclAppScanOnCloudCredential,
    HclAppScanOnCloudCredential_OAuthClient,
    HclAppScanOnCloudCredential_OAuthClientId,
)
from .hcl_app_scan_on_cloud_url import HclAppScanOnCloudUrl
from .identity_entra_id import IdentityEntraId
from .identity_google import IdentityGoogle
from .identity_okta import IdentityOkta
from .identity_ping_one import IdentityPingOne
from .incident_io_credential import (
    IncidentIoCredential,
    IncidentIoCredential_Token,
    IncidentIoCredential_TokenId,
)
from .incident_response_incident_io import IncidentResponseIncidentIo
from .incident_response_pager_duty import IncidentResponsePagerDuty
from .ivanti_credential import (
    IvantiCredential,
    IvantiCredential_OAuthClient,
    IvantiCredential_OAuthClientId,
)
from .ivanti_credential_ticketing import (
    IvantiCredentialTicketing,
    IvantiCredentialTicketing_Token,
    IvantiCredentialTicketing_TokenId,
)
from .jira_credential import (
    JiraCredential,
    JiraCredential_Basic,
    JiraCredential_BasicId,
)
from .malwarebytes_credential import (
    MalwarebytesCredential,
    MalwarebytesCredential_OAuthClient,
    MalwarebytesCredential_OAuthClientId,
)
from .notifications_jira import NotificationsJira
from .notifications_mock import NotificationsMock
from .notifications_slack import NotificationsSlack
from .notifications_slack_webhook import NotificationsSlackWebhook
from .notifications_teams import NotificationsTeams
from .nozomi_vantage_credential import (
    NozomiVantageCredential,
    NozomiVantageCredential_Basic,
    NozomiVantageCredential_BasicId,
)
from .nucleus_credential import (
    NucleusCredential,
    NucleusCredential_Token,
    NucleusCredential_TokenId,
)
from .okta_credential import (
    OktaCredential,
    OktaCredential_OAuthClient,
    OktaCredential_OAuthClientId,
    OktaCredential_Token,
    OktaCredential_TokenId,
)
from .open_search_credential import (
    OpenSearchCredential,
    OpenSearchCredential_Basic,
    OpenSearchCredential_BasicId,
)
from .open_text_application_security_credential import (
    OpenTextApplicationSecurityCredential,
    OpenTextApplicationSecurityCredential_Token,
    OpenTextApplicationSecurityCredential_TokenId,
)
from .open_text_core_application_security_credential import (
    OpenTextCoreApplicationSecurityCredential,
    OpenTextCoreApplicationSecurityCredential_OAuthClient,
    OpenTextCoreApplicationSecurityCredential_OAuthClientId,
)
from .open_text_core_application_security_url import OpenTextCoreApplicationSecurityUrl
from .pager_duty_credential import (
    PagerDutyCredential,
    PagerDutyCredential_Token,
    PagerDutyCredential_TokenId,
)
from .palo_alto_credential import (
    PaloAltoCredential,
    PaloAltoCredential_Basic,
    PaloAltoCredential_BasicId,
)
from .ping_one_apiurl import PingOneApiurl
from .ping_one_auth_url import PingOneAuthUrl
from .ping_one_credential import (
    PingOneCredential,
    PingOneCredential_Token,
    PingOneCredential_TokenId,
)
from .provider_config import (
    ProviderConfig,
    ProviderConfig_AppsecAmazonInspector,
    ProviderConfig_AppsecGitlab,
    ProviderConfig_AppsecHclAppscanOnCloud,
    ProviderConfig_AppsecOpentextApplicationSecurity,
    ProviderConfig_AppsecOpentextCoreApplicationSecurity,
    ProviderConfig_AppsecOpentextCoreApplicationSecurityMock,
    ProviderConfig_AssetsArmisCentrix,
    ProviderConfig_AssetsArmisCentrixMock,
    ProviderConfig_AssetsAxonius,
    ProviderConfig_AssetsAxoniusMock,
    ProviderConfig_AssetsClarotyXdome,
    ProviderConfig_AssetsCrowdstrike,
    ProviderConfig_AssetsCrowdstrikeMock,
    ProviderConfig_AssetsIvantiNeurons,
    ProviderConfig_AssetsNozomiVantage,
    ProviderConfig_AssetsNozomiVantageMock,
    ProviderConfig_AssetsQualysCloud,
    ProviderConfig_AssetsQualysCloudMock,
    ProviderConfig_AssetsServicenow,
    ProviderConfig_AssetsServicenowMock,
    ProviderConfig_AssetsSevco,
    ProviderConfig_AssetsSevcoMock,
    ProviderConfig_AssetsTaniumCloud,
    ProviderConfig_AssetsTaniumCloudMock,
    ProviderConfig_CloudsecurityAws,
    ProviderConfig_CloudsecurityCrowdstrike,
    ProviderConfig_CloudsecurityDefender,
    ProviderConfig_CloudsecurityPaloalto,
    ProviderConfig_CustomSynqly,
    ProviderConfig_EdrCrowdstrike,
    ProviderConfig_EdrCrowdstrikeMock,
    ProviderConfig_EdrDefender,
    ProviderConfig_EdrMalwarebytes,
    ProviderConfig_EdrSentinelone,
    ProviderConfig_EdrSophos,
    ProviderConfig_EdrTanium,
    ProviderConfig_IdentityEntraId,
    ProviderConfig_IdentityGoogle,
    ProviderConfig_IdentityOkta,
    ProviderConfig_IdentityPingone,
    ProviderConfig_IncidentresponseIncidentio,
    ProviderConfig_IncidentresponsePagerduty,
    ProviderConfig_NotificationsJira,
    ProviderConfig_NotificationsMockNotifications,
    ProviderConfig_NotificationsSlack,
    ProviderConfig_NotificationsSlackWebhook,
    ProviderConfig_NotificationsTeams,
    ProviderConfig_SiemCrowdstrike,
    ProviderConfig_SiemElasticsearch,
    ProviderConfig_SiemGoogleChronicle,
    ProviderConfig_SiemGoogleSecurityOperations,
    ProviderConfig_SiemMockSiem,
    ProviderConfig_SiemOpensearch,
    ProviderConfig_SiemQRadar,
    ProviderConfig_SiemRapid7Insightidr,
    ProviderConfig_SiemSentinel,
    ProviderConfig_SiemSplunk,
    ProviderConfig_SiemSumoLogic,
    ProviderConfig_SinkAwsS3,
    ProviderConfig_SinkAwsSecurityLake,
    ProviderConfig_SinkAwsSqs,
    ProviderConfig_SinkAzureBlob,
    ProviderConfig_SinkAzureMonitorLogs,
    ProviderConfig_SinkCrowdstrikeHec,
    ProviderConfig_SinkElasticsearch,
    ProviderConfig_SinkGcs,
    ProviderConfig_SinkGoogleSecOps,
    ProviderConfig_SinkGoogleSecurityOperations,
    ProviderConfig_SinkMockSink,
    ProviderConfig_SinkOpensearch,
    ProviderConfig_SinkQRadar,
    ProviderConfig_SinkSplunk,
    ProviderConfig_SinkSumoLogic,
    ProviderConfig_StorageAwsS3,
    ProviderConfig_StorageAzureBlob,
    ProviderConfig_StorageGcs,
    ProviderConfig_StorageMockStorage,
    ProviderConfig_TicketingAutotask,
    ProviderConfig_TicketingFreshdesk,
    ProviderConfig_TicketingIvanti,
    ProviderConfig_TicketingJira,
    ProviderConfig_TicketingJiraServiceManagement,
    ProviderConfig_TicketingMockTicketing,
    ProviderConfig_TicketingPagerduty,
    ProviderConfig_TicketingPagerdutyMock,
    ProviderConfig_TicketingServicenow,
    ProviderConfig_TicketingServicenowSir,
    ProviderConfig_TicketingTorq,
    ProviderConfig_TicketingZendesk,
    ProviderConfig_VulnerabilitiesCrowdstrike,
    ProviderConfig_VulnerabilitiesCrowdstrikeMock,
    ProviderConfig_VulnerabilitiesNucleus,
    ProviderConfig_VulnerabilitiesQualysCloud,
    ProviderConfig_VulnerabilitiesQualysCloudMock,
    ProviderConfig_VulnerabilitiesRapid7InsightCloud,
    ProviderConfig_VulnerabilitiesRapid7InsightCloudMock,
    ProviderConfig_VulnerabilitiesServicenowVr,
    ProviderConfig_VulnerabilitiesTaniumCloud,
    ProviderConfig_VulnerabilitiesTaniumCloudMock,
    ProviderConfig_VulnerabilitiesTenableCloud,
)
from .provider_config_id import ProviderConfigId
from .q_radar_credential import (
    QRadarCredential,
    QRadarCredential_Token,
    QRadarCredential_TokenId,
)
from .qualys_cloud_credential import (
    QualysCloudCredential,
    QualysCloudCredential_Basic,
    QualysCloudCredential_BasicId,
)
from .rapid_7_insight_cloud_credential import (
    Rapid7InsightCloudCredential,
    Rapid7InsightCloudCredential_Token,
    Rapid7InsightCloudCredential_TokenId,
)
from .sentinel_credential import (
    SentinelCredential,
    SentinelCredential_OAuthClient,
    SentinelCredential_OAuthClientId,
)
from .sentinel_one_credential import (
    SentinelOneCredential,
    SentinelOneCredential_Token,
    SentinelOneCredential_TokenId,
)
from .sentinel_one_edr_events_credential import (
    SentinelOneEdrEventsCredential,
    SentinelOneEdrEventsCredential_Token,
    SentinelOneEdrEventsCredential_TokenId,
)
from .service_now_credential import (
    ServiceNowCredential,
    ServiceNowCredential_Basic,
    ServiceNowCredential_BasicId,
    ServiceNowCredential_Token,
    ServiceNowCredential_TokenId,
)
from .sevco_credential import (
    SevcoCredential,
    SevcoCredential_Token,
    SevcoCredential_TokenId,
)
from .siem_crowdstrike import SiemCrowdstrike
from .siem_elasticsearch import SiemElasticsearch
from .siem_google_chronicle import SiemGoogleChronicle
from .siem_google_security_operations import SiemGoogleSecurityOperations
from .siem_mock import SiemMock
from .siem_open_search import SiemOpenSearch
from .siem_q_radar import SiemQRadar
from .siem_rapid_7_insight_idr import SiemRapid7InsightIdr
from .siem_sentinel import SiemSentinel
from .siem_splunk import SiemSplunk
from .siem_sumo_logic import SiemSumoLogic
from .sink_aws_s_3 import SinkAwsS3
from .sink_aws_security_lake import SinkAwsSecurityLake
from .sink_aws_sqs import SinkAwsSqs
from .sink_azure_blob import SinkAzureBlob
from .sink_azure_monitor_logs import SinkAzureMonitorLogs
from .sink_crowdstrike_hec import SinkCrowdstrikeHec
from .sink_elasticsearch import SinkElasticsearch
from .sink_gcs import SinkGcs
from .sink_google_sec_ops import SinkGoogleSecOps
from .sink_google_security_operations import SinkGoogleSecurityOperations
from .sink_mock import SinkMock
from .sink_open_search import SinkOpenSearch
from .sink_q_radar import SinkQRadar
from .sink_splunk import SinkSplunk
from .sink_sumo_logic import SinkSumoLogic
from .slack_credential import (
    SlackCredential,
    SlackCredential_Token,
    SlackCredential_TokenId,
)
from .slack_webhook_credential import (
    SlackWebhookCredential,
    SlackWebhookCredential_Secret,
    SlackWebhookCredential_SecretId,
)
from .sophos_credential import (
    SophosCredential,
    SophosCredential_OAuthClient,
    SophosCredential_OAuthClientId,
)
from .splunk_hec_token import (
    SplunkHecToken,
    SplunkHecToken_Token,
    SplunkHecToken_TokenId,
)
from .splunk_search_credential import (
    SplunkSearchCredential,
    SplunkSearchCredential_Token,
    SplunkSearchCredential_TokenId,
)
from .storage_aws_s_3 import StorageAwsS3
from .storage_azure_blob import StorageAzureBlob
from .storage_gcs import StorageGcs
from .storage_mock import StorageMock
from .sumo_logic_collection_url import (
    SumoLogicCollectionUrl,
    SumoLogicCollectionUrl_Secret,
    SumoLogicCollectionUrl_SecretId,
)
from .sumo_logic_credential import (
    SumoLogicCredential,
    SumoLogicCredential_Basic,
    SumoLogicCredential_BasicId,
)
from .tanium_cloud_credential import (
    TaniumCloudCredential,
    TaniumCloudCredential_Token,
    TaniumCloudCredential_TokenId,
)
from .teams_credential import (
    TeamsCredential,
    TeamsCredential_OAuthClient,
    TeamsCredential_OAuthClientId,
    TeamsCredential_WebhookUrl,
    TeamsCredential_WebhookUrlId,
)
from .tenable_cloud_credential import (
    TenableCloudCredential,
    TenableCloudCredential_Token,
    TenableCloudCredential_TokenId,
)
from .ticketing_autotask import TicketingAutotask
from .ticketing_freshdesk import TicketingFreshdesk
from .ticketing_ivanti import TicketingIvanti
from .ticketing_jira import TicketingJira
from .ticketing_jira_service_management import TicketingJiraServiceManagement
from .ticketing_mock import TicketingMock
from .ticketing_pager_duty import TicketingPagerDuty
from .ticketing_pager_duty_mock import TicketingPagerDutyMock
from .ticketing_pagerduty_dataset import TicketingPagerdutyDataset
from .ticketing_service_now import TicketingServiceNow
from .ticketing_service_now_sir import TicketingServiceNowSir
from .ticketing_torq import TicketingTorq
from .ticketing_zendesk import TicketingZendesk
from .torq_credential import (
    TorqCredential,
    TorqCredential_OAuthClient,
    TorqCredential_OAuthClientId,
)
from .vulnerabilities_crowd_strike import VulnerabilitiesCrowdStrike
from .vulnerabilities_crowd_strike_dataset import VulnerabilitiesCrowdStrikeDataset
from .vulnerabilities_crowd_strike_mock import VulnerabilitiesCrowdStrikeMock
from .vulnerabilities_nucleus import VulnerabilitiesNucleus
from .vulnerabilities_qualys_cloud import VulnerabilitiesQualysCloud
from .vulnerabilities_qualys_cloud_dataset import VulnerabilitiesQualysCloudDataset
from .vulnerabilities_qualys_cloud_mock import VulnerabilitiesQualysCloudMock
from .vulnerabilities_rapid_7_insight_cloud import VulnerabilitiesRapid7InsightCloud
from .vulnerabilities_rapid_7_insight_cloud_dataset import (
    VulnerabilitiesRapid7InsightCloudDataset,
)
from .vulnerabilities_rapid_7_insight_cloud_mock import (
    VulnerabilitiesRapid7InsightCloudMock,
)
from .vulnerabilities_service_now import VulnerabilitiesServiceNow
from .vulnerabilities_tanium_cloud import VulnerabilitiesTaniumCloud
from .vulnerabilities_tanium_cloud_dataset import VulnerabilitiesTaniumCloudDataset
from .vulnerabilities_tanium_cloud_mock import VulnerabilitiesTaniumCloudMock
from .vulnerabilities_tenable_cloud import VulnerabilitiesTenableCloud
from .zendesk_credential import (
    ZendeskCredential,
    ZendeskCredential_Basic,
    ZendeskCredential_BasicId,
)

__all__ = [
    "AppsecAmazonInspector",
    "AppsecGitLab",
    "AppsecHclAppScanOnCloud",
    "AppsecOpenTextApplicationSecurity",
    "AppsecOpenTextCoreApplicationSecurity",
    "AppsecOpenTextCoreApplicationSecurityMock",
    "AppsecOpentextCoreApplicationSecurityDataset",
    "ArmisCredential",
    "ArmisCredential_Token",
    "ArmisCredential_TokenId",
    "AssetsArmisCentrix",
    "AssetsArmisCentrixMock",
    "AssetsArmisDataset",
    "AssetsAxonius",
    "AssetsAxoniusDataset",
    "AssetsAxoniusMock",
    "AssetsClarotyXdome",
    "AssetsCrowdStrike",
    "AssetsCrowdStrikeDataset",
    "AssetsCrowdStrikeMock",
    "AssetsIvantiNeurons",
    "AssetsNozomiVantage",
    "AssetsNozomiVantageDataset",
    "AssetsNozomiVantageMock",
    "AssetsQualysCloud",
    "AssetsQualysCloudDataset",
    "AssetsQualysCloudMock",
    "AssetsServiceNow",
    "AssetsServiceNowDataset",
    "AssetsServiceNowMock",
    "AssetsSevco",
    "AssetsSevcoDataset",
    "AssetsSevcoMock",
    "AssetsTaniumCloud",
    "AssetsTaniumCloudDataset",
    "AssetsTaniumCloudMock",
    "AutotaskApiIntegrationCodeCredential",
    "AutotaskApiIntegrationCodeCredential_Secret",
    "AutotaskApiIntegrationCodeCredential_SecretId",
    "AutotaskSecretCredential",
    "AutotaskSecretCredential_Secret",
    "AutotaskSecretCredential_SecretId",
    "AwsProviderCredential",
    "AwsProviderCredential_Aws",
    "AwsProviderCredential_AwsId",
    "AwsRegion",
    "AwsS3Credential",
    "AwsS3Credential_Aws",
    "AwsS3Credential_AwsId",
    "AwsSecurityLakeCredential",
    "AwsSecurityLakeCredential_Aws",
    "AwsSecurityLakeCredential_AwsId",
    "AwsSqsCredential",
    "AwsSqsCredential_Aws",
    "AwsSqsCredential_AwsId",
    "AxoniusCredential",
    "AxoniusCredential_Basic",
    "AxoniusCredential_BasicId",
    "AzureBlobCredential",
    "AzureBlobCredential_Token",
    "AzureBlobCredential_TokenId",
    "AzureMonitorLogsCredential",
    "AzureMonitorLogsCredential_Token",
    "AzureMonitorLogsCredential_TokenId",
    "ClarotyApiurl",
    "ClarotyCredential",
    "ClarotyCredential_Token",
    "ClarotyCredential_TokenId",
    "CloudSecurityAws",
    "CloudSecurityCrowdStrike",
    "CloudSecurityDefender",
    "CloudSecurityPaloAlto",
    "CrowdStrikeCredential",
    "CrowdStrikeCredential_OAuthClient",
    "CrowdStrikeCredential_OAuthClientId",
    "CrowdstrikeHecCredential",
    "CrowdstrikeHecCredential_Token",
    "CrowdstrikeHecCredential_TokenId",
    "CustomCredential",
    "CustomCredential_Aws",
    "CustomCredential_AwsId",
    "CustomCredential_Basic",
    "CustomCredential_BasicId",
    "CustomCredential_OAuthClient",
    "CustomCredential_OAuthClientId",
    "CustomCredential_Secret",
    "CustomCredential_SecretId",
    "CustomCredential_Token",
    "CustomCredential_TokenId",
    "CustomEndpoint",
    "CustomFieldMapping",
    "CustomSynqly",
    "DefenderCredential",
    "DefenderCredential_OAuthClient",
    "DefenderCredential_OAuthClientId",
    "EdrCrowdStrike",
    "EdrCrowdStrikeDataset",
    "EdrCrowdStrikeMock",
    "EdrDefender",
    "EdrMalwarebytes",
    "EdrSentinelOne",
    "EdrSophos",
    "EdrTanium",
    "ElasticsearchAuthOptions",
    "ElasticsearchCredential",
    "ElasticsearchCredential_Basic",
    "ElasticsearchCredential_BasicId",
    "ElasticsearchCredential_OAuthClient",
    "ElasticsearchCredential_OAuthClientId",
    "ElasticsearchCredential_Token",
    "ElasticsearchCredential_TokenId",
    "ElasticsearchSharedSecret",
    "ElasticsearchSharedSecret_Secret",
    "ElasticsearchSharedSecret_SecretId",
    "EntraIdCredential",
    "EntraIdCredential_OAuthClient",
    "EntraIdCredential_OAuthClientId",
    "FreshdeskCredential",
    "FreshdeskCredential_Token",
    "FreshdeskCredential_TokenId",
    "GcsCredential",
    "GcsCredential_Aws",
    "GcsCredential_AwsId",
    "GcsJsonCredential",
    "GcsJsonCredential_Token",
    "GcsJsonCredential_TokenId",
    "GitLabCredential",
    "GitLabCredential_Token",
    "GitLabCredential_TokenId",
    "GoogleChronicleCredential",
    "GoogleChronicleCredential_OAuthClient",
    "GoogleChronicleCredential_OAuthClientId",
    "GoogleCredential",
    "GoogleCredential_OAuthClient",
    "GoogleCredential_OAuthClientId",
    "GoogleServiceAccountCredential",
    "GoogleServiceAccountCredential_OAuthClient",
    "GoogleServiceAccountCredential_OAuthClientId",
    "HclAppScanOnCloudCredential",
    "HclAppScanOnCloudCredential_OAuthClient",
    "HclAppScanOnCloudCredential_OAuthClientId",
    "HclAppScanOnCloudUrl",
    "IdentityEntraId",
    "IdentityGoogle",
    "IdentityOkta",
    "IdentityPingOne",
    "IncidentIoCredential",
    "IncidentIoCredential_Token",
    "IncidentIoCredential_TokenId",
    "IncidentResponseIncidentIo",
    "IncidentResponsePagerDuty",
    "IvantiCredential",
    "IvantiCredentialTicketing",
    "IvantiCredentialTicketing_Token",
    "IvantiCredentialTicketing_TokenId",
    "IvantiCredential_OAuthClient",
    "IvantiCredential_OAuthClientId",
    "JiraCredential",
    "JiraCredential_Basic",
    "JiraCredential_BasicId",
    "MalwarebytesCredential",
    "MalwarebytesCredential_OAuthClient",
    "MalwarebytesCredential_OAuthClientId",
    "NotificationsJira",
    "NotificationsMock",
    "NotificationsSlack",
    "NotificationsSlackWebhook",
    "NotificationsTeams",
    "NozomiVantageCredential",
    "NozomiVantageCredential_Basic",
    "NozomiVantageCredential_BasicId",
    "NucleusCredential",
    "NucleusCredential_Token",
    "NucleusCredential_TokenId",
    "OktaCredential",
    "OktaCredential_OAuthClient",
    "OktaCredential_OAuthClientId",
    "OktaCredential_Token",
    "OktaCredential_TokenId",
    "OpenSearchCredential",
    "OpenSearchCredential_Basic",
    "OpenSearchCredential_BasicId",
    "OpenTextApplicationSecurityCredential",
    "OpenTextApplicationSecurityCredential_Token",
    "OpenTextApplicationSecurityCredential_TokenId",
    "OpenTextCoreApplicationSecurityCredential",
    "OpenTextCoreApplicationSecurityCredential_OAuthClient",
    "OpenTextCoreApplicationSecurityCredential_OAuthClientId",
    "OpenTextCoreApplicationSecurityUrl",
    "PagerDutyCredential",
    "PagerDutyCredential_Token",
    "PagerDutyCredential_TokenId",
    "PaloAltoCredential",
    "PaloAltoCredential_Basic",
    "PaloAltoCredential_BasicId",
    "PingOneApiurl",
    "PingOneAuthUrl",
    "PingOneCredential",
    "PingOneCredential_Token",
    "PingOneCredential_TokenId",
    "ProviderConfig",
    "ProviderConfigId",
    "ProviderConfig_AppsecAmazonInspector",
    "ProviderConfig_AppsecGitlab",
    "ProviderConfig_AppsecHclAppscanOnCloud",
    "ProviderConfig_AppsecOpentextApplicationSecurity",
    "ProviderConfig_AppsecOpentextCoreApplicationSecurity",
    "ProviderConfig_AppsecOpentextCoreApplicationSecurityMock",
    "ProviderConfig_AssetsArmisCentrix",
    "ProviderConfig_AssetsArmisCentrixMock",
    "ProviderConfig_AssetsAxonius",
    "ProviderConfig_AssetsAxoniusMock",
    "ProviderConfig_AssetsClarotyXdome",
    "ProviderConfig_AssetsCrowdstrike",
    "ProviderConfig_AssetsCrowdstrikeMock",
    "ProviderConfig_AssetsIvantiNeurons",
    "ProviderConfig_AssetsNozomiVantage",
    "ProviderConfig_AssetsNozomiVantageMock",
    "ProviderConfig_AssetsQualysCloud",
    "ProviderConfig_AssetsQualysCloudMock",
    "ProviderConfig_AssetsServicenow",
    "ProviderConfig_AssetsServicenowMock",
    "ProviderConfig_AssetsSevco",
    "ProviderConfig_AssetsSevcoMock",
    "ProviderConfig_AssetsTaniumCloud",
    "ProviderConfig_AssetsTaniumCloudMock",
    "ProviderConfig_CloudsecurityAws",
    "ProviderConfig_CloudsecurityCrowdstrike",
    "ProviderConfig_CloudsecurityDefender",
    "ProviderConfig_CloudsecurityPaloalto",
    "ProviderConfig_CustomSynqly",
    "ProviderConfig_EdrCrowdstrike",
    "ProviderConfig_EdrCrowdstrikeMock",
    "ProviderConfig_EdrDefender",
    "ProviderConfig_EdrMalwarebytes",
    "ProviderConfig_EdrSentinelone",
    "ProviderConfig_EdrSophos",
    "ProviderConfig_EdrTanium",
    "ProviderConfig_IdentityEntraId",
    "ProviderConfig_IdentityGoogle",
    "ProviderConfig_IdentityOkta",
    "ProviderConfig_IdentityPingone",
    "ProviderConfig_IncidentresponseIncidentio",
    "ProviderConfig_IncidentresponsePagerduty",
    "ProviderConfig_NotificationsJira",
    "ProviderConfig_NotificationsMockNotifications",
    "ProviderConfig_NotificationsSlack",
    "ProviderConfig_NotificationsSlackWebhook",
    "ProviderConfig_NotificationsTeams",
    "ProviderConfig_SiemCrowdstrike",
    "ProviderConfig_SiemElasticsearch",
    "ProviderConfig_SiemGoogleChronicle",
    "ProviderConfig_SiemGoogleSecurityOperations",
    "ProviderConfig_SiemMockSiem",
    "ProviderConfig_SiemOpensearch",
    "ProviderConfig_SiemQRadar",
    "ProviderConfig_SiemRapid7Insightidr",
    "ProviderConfig_SiemSentinel",
    "ProviderConfig_SiemSplunk",
    "ProviderConfig_SiemSumoLogic",
    "ProviderConfig_SinkAwsS3",
    "ProviderConfig_SinkAwsSecurityLake",
    "ProviderConfig_SinkAwsSqs",
    "ProviderConfig_SinkAzureBlob",
    "ProviderConfig_SinkAzureMonitorLogs",
    "ProviderConfig_SinkCrowdstrikeHec",
    "ProviderConfig_SinkElasticsearch",
    "ProviderConfig_SinkGcs",
    "ProviderConfig_SinkGoogleSecOps",
    "ProviderConfig_SinkGoogleSecurityOperations",
    "ProviderConfig_SinkMockSink",
    "ProviderConfig_SinkOpensearch",
    "ProviderConfig_SinkQRadar",
    "ProviderConfig_SinkSplunk",
    "ProviderConfig_SinkSumoLogic",
    "ProviderConfig_StorageAwsS3",
    "ProviderConfig_StorageAzureBlob",
    "ProviderConfig_StorageGcs",
    "ProviderConfig_StorageMockStorage",
    "ProviderConfig_TicketingAutotask",
    "ProviderConfig_TicketingFreshdesk",
    "ProviderConfig_TicketingIvanti",
    "ProviderConfig_TicketingJira",
    "ProviderConfig_TicketingJiraServiceManagement",
    "ProviderConfig_TicketingMockTicketing",
    "ProviderConfig_TicketingPagerduty",
    "ProviderConfig_TicketingPagerdutyMock",
    "ProviderConfig_TicketingServicenow",
    "ProviderConfig_TicketingServicenowSir",
    "ProviderConfig_TicketingTorq",
    "ProviderConfig_TicketingZendesk",
    "ProviderConfig_VulnerabilitiesCrowdstrike",
    "ProviderConfig_VulnerabilitiesCrowdstrikeMock",
    "ProviderConfig_VulnerabilitiesNucleus",
    "ProviderConfig_VulnerabilitiesQualysCloud",
    "ProviderConfig_VulnerabilitiesQualysCloudMock",
    "ProviderConfig_VulnerabilitiesRapid7InsightCloud",
    "ProviderConfig_VulnerabilitiesRapid7InsightCloudMock",
    "ProviderConfig_VulnerabilitiesServicenowVr",
    "ProviderConfig_VulnerabilitiesTaniumCloud",
    "ProviderConfig_VulnerabilitiesTaniumCloudMock",
    "ProviderConfig_VulnerabilitiesTenableCloud",
    "QRadarCredential",
    "QRadarCredential_Token",
    "QRadarCredential_TokenId",
    "QualysCloudCredential",
    "QualysCloudCredential_Basic",
    "QualysCloudCredential_BasicId",
    "Rapid7InsightCloudCredential",
    "Rapid7InsightCloudCredential_Token",
    "Rapid7InsightCloudCredential_TokenId",
    "SentinelCredential",
    "SentinelCredential_OAuthClient",
    "SentinelCredential_OAuthClientId",
    "SentinelOneCredential",
    "SentinelOneCredential_Token",
    "SentinelOneCredential_TokenId",
    "SentinelOneEdrEventsCredential",
    "SentinelOneEdrEventsCredential_Token",
    "SentinelOneEdrEventsCredential_TokenId",
    "ServiceNowCredential",
    "ServiceNowCredential_Basic",
    "ServiceNowCredential_BasicId",
    "ServiceNowCredential_Token",
    "ServiceNowCredential_TokenId",
    "SevcoCredential",
    "SevcoCredential_Token",
    "SevcoCredential_TokenId",
    "SiemCrowdstrike",
    "SiemElasticsearch",
    "SiemGoogleChronicle",
    "SiemGoogleSecurityOperations",
    "SiemMock",
    "SiemOpenSearch",
    "SiemQRadar",
    "SiemRapid7InsightIdr",
    "SiemSentinel",
    "SiemSplunk",
    "SiemSumoLogic",
    "SinkAwsS3",
    "SinkAwsSecurityLake",
    "SinkAwsSqs",
    "SinkAzureBlob",
    "SinkAzureMonitorLogs",
    "SinkCrowdstrikeHec",
    "SinkElasticsearch",
    "SinkGcs",
    "SinkGoogleSecOps",
    "SinkGoogleSecurityOperations",
    "SinkMock",
    "SinkOpenSearch",
    "SinkQRadar",
    "SinkSplunk",
    "SinkSumoLogic",
    "SlackCredential",
    "SlackCredential_Token",
    "SlackCredential_TokenId",
    "SlackWebhookCredential",
    "SlackWebhookCredential_Secret",
    "SlackWebhookCredential_SecretId",
    "SophosCredential",
    "SophosCredential_OAuthClient",
    "SophosCredential_OAuthClientId",
    "SplunkHecToken",
    "SplunkHecToken_Token",
    "SplunkHecToken_TokenId",
    "SplunkSearchCredential",
    "SplunkSearchCredential_Token",
    "SplunkSearchCredential_TokenId",
    "StorageAwsS3",
    "StorageAzureBlob",
    "StorageGcs",
    "StorageMock",
    "SumoLogicCollectionUrl",
    "SumoLogicCollectionUrl_Secret",
    "SumoLogicCollectionUrl_SecretId",
    "SumoLogicCredential",
    "SumoLogicCredential_Basic",
    "SumoLogicCredential_BasicId",
    "TaniumCloudCredential",
    "TaniumCloudCredential_Token",
    "TaniumCloudCredential_TokenId",
    "TeamsCredential",
    "TeamsCredential_OAuthClient",
    "TeamsCredential_OAuthClientId",
    "TeamsCredential_WebhookUrl",
    "TeamsCredential_WebhookUrlId",
    "TenableCloudCredential",
    "TenableCloudCredential_Token",
    "TenableCloudCredential_TokenId",
    "TicketingAutotask",
    "TicketingFreshdesk",
    "TicketingIvanti",
    "TicketingJira",
    "TicketingJiraServiceManagement",
    "TicketingMock",
    "TicketingPagerDuty",
    "TicketingPagerDutyMock",
    "TicketingPagerdutyDataset",
    "TicketingServiceNow",
    "TicketingServiceNowSir",
    "TicketingTorq",
    "TicketingZendesk",
    "TorqCredential",
    "TorqCredential_OAuthClient",
    "TorqCredential_OAuthClientId",
    "VulnerabilitiesCrowdStrike",
    "VulnerabilitiesCrowdStrikeDataset",
    "VulnerabilitiesCrowdStrikeMock",
    "VulnerabilitiesNucleus",
    "VulnerabilitiesQualysCloud",
    "VulnerabilitiesQualysCloudDataset",
    "VulnerabilitiesQualysCloudMock",
    "VulnerabilitiesRapid7InsightCloud",
    "VulnerabilitiesRapid7InsightCloudDataset",
    "VulnerabilitiesRapid7InsightCloudMock",
    "VulnerabilitiesServiceNow",
    "VulnerabilitiesTaniumCloud",
    "VulnerabilitiesTaniumCloudDataset",
    "VulnerabilitiesTaniumCloudMock",
    "VulnerabilitiesTenableCloud",
    "ZendeskCredential",
    "ZendeskCredential_Basic",
    "ZendeskCredential_BasicId",
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .account import Account
import pydantic
from ...base.types.email_address import EmailAddress
from .group import Group
from .ldap_person import LdapPerson
from .user_mfa_status_id import UserMfaStatusId
from ...base.types.user_name import UserName
from .organization import Organization
from .user_risk_level_id import UserRiskLevelId
from .user_type_id import UserTypeId
from .user_user_status_id import UserUserStatusId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class User(UncheckedBaseModel):
    """
    The User object describes the characteristics of a user/person or a security principal. Defined by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:UserAccount/'>d3f:UserAccount</a>.
    """

    account: typing.Optional[Account] = pydantic.Field(default=None)
    """
    The user's account or the account associated with the user.
    """

    credential_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the user's credential. For example, AWS Access Key ID.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain where the user is defined. For example: the LDAP or Active Directory domain.
    """

    email_addr: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The user's primary email address.
    """

    full_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full name of the person, as per the LDAP Common Name attribute (cn).
    """

    groups: typing.Optional[typing.List[Group]] = pydantic.Field(default=None)
    """
    The administrative groups to which the user belongs.
    """

    has_mfa: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The user has a multi-factor or secondary-factor device assigned.
    """

    ldap_person: typing.Optional[LdapPerson] = pydantic.Field(default=None)
    """
    The additional LDAP attributes that describe a person.
    """

    mfa_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The multi-factor authentication status, normalized to the caption of the mfa_status_id value. In the case of 'Other', it is defined by the data source.
    """

    mfa_status_id: typing.Optional[UserMfaStatusId] = pydantic.Field(default=None)
    """
    The normalized identifier of the user's multi-factor authentication status.
    """

    name: typing.Optional[UserName] = pydantic.Field(default=None)
    """
    The username. For example, <code>janedoe1</code>.
    """

    org: typing.Optional[Organization] = pydantic.Field(default=None)
    """
    Organization and org unit related to the user.
    """

    privileges: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The user's privileges.
    """

    risk_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    The risk level, normalized to the caption of the risk_level_id value.
    """

    risk_level_id: typing.Optional[UserRiskLevelId] = pydantic.Field(default=None)
    """
    The normalized risk level id.
    """

    risk_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The risk score as reported by the event source.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the user. For example, System, AWS IAM User, etc.
    """

    type_id: typing.Optional[UserTypeId] = pydantic.Field(default=None)
    """
    The account type identifier.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique user identifier. For example, the Windows user SID, ActiveDirectory DN or AWS user ARN.
    """

    uid_alt: typing.Optional[str] = pydantic.Field(default=None)
    """
    The alternate user identifier. For example, the Active Directory user GUID or AWS user Principal ID.
    """

    user_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The user status, normalized to the caption of the user_status_id value. In the case of 'Other', it is defined by the data source.
    """

    user_status_id: typing.Optional[UserUserStatusId] = pydantic.Field(default=None)
    """
    The normalized identifier of the user's status.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow
